import os
import platform
from setuptools import setup, find_packages
from setuptools.command.install import install
import subprocess

class CustomInstallCommand(install):
    """Customized setuptools install command to convert README to man page and install it."""
    def run(self):
        if platform.system() != 'Windows':
            # Check if pandoc is available
            try:
                subprocess.run(['pandoc', '--version'], check=True)
            except subprocess.CalledProcessError:
                print("Pandoc is not available. Skipping man page generation.")
                install.run(self)
                return

            # Convert README.md to man page using pandoc
            subprocess.run(['pandoc', 'README.md', '-s', '-t', 'man', '-o', 'nomad-media-cli.1'])
            
            # Install the man page
            man_dir = '/usr/share/man/man1'
            if not os.path.exists(man_dir):
                os.makedirs(man_dir)
            subprocess.run(['sudo', 'cp', 'nomad-media-cli.1', man_dir])
            subprocess.run(['sudo', 'mandb'])
        else:
            print("Skipping man page generation on Windows.")

        # Run the standard install process
        install.run(self)

setup(
    name='nomad_media_cli',
    version='0.0.1a20',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'click',
        'platformdirs',
        'nomad-media-pip==0.1.2a11',
    ],
    entry_points={
        'console_scripts': [
            'nomad-media-cli=nomad_media_cli.cli:cli',
        ],
    },
    cmdclass={
        'install': CustomInstallCommand,
    },
    author='Nomad Media',
    description='Nomad Media CLI',
    classifiers=[
        'Programming Language :: Python :: 3.12',
    ],
    python_requires='>=3.12',
)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--schedule-id", required=True, help="The ID of the schedule the intelligent playlist is to be updated.")
@click.option("--collections", required=False, help="The collections of the intelligent playlist in JSON list format.")
@click.option("--end-search-date", required=False, help="The end search date of the intelligent playlist. Format: yyyy-MM-dd.THH:MM:SS.FFFZ.")
@click.option("--end-search-duration-in-minutes", required=False, type=click.INT, help="The end search duration in minutes of the intelligent playlist.")
@click.option("--name", required=False, help="The name of the intelligent playlist.")
@click.option("--related-contents", required=False, help="The related content of the intelligent playlist in JSON list format.")
@click.option("--search-date", required=False, help="The search date of the intelligent playlist. Format: yyyy-MM-dd.THH:MM:SS.FFFZ.")
@click.option("--search-duration-in-minutes", required=False, type=click.INT, help="The search duration in minutes of the intelligent playlist.")
@click.option("--search-filter-type", required=False, type=click.INT, help="The search filter type of the intelligent playlist. Values: Random: 1, Random within a Date Range: 2, Newest: 3, Newest Not Played: 4")
@click.option("--tags", required=False, help="The tags of the intelligent playlist in JSON list format.")
@click.option("--thumbnail-asset", required=False, help="The thumbnail asset of the intelligent playlist in JSON dict format.")
@click.pass_context
def update_intelligent_playlist(ctx, schedule_id, collections, end_search_date, end_search_duration_in_minutes, name, related_contents, search_date, search_duration_in_minutes, search_filter_type, tags, thumbnail_asset):
    """Update intelligent playlist"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.update_intelligent_playlist(
            schedule_id,
            json.loads(collections) if collections else None,
            end_search_date,
            end_search_duration_in_minutes,
            name,
            json.loads(related_contents) if related_contents else None,
            search_date,
            search_duration_in_minutes,
            search_filter_type,
            json.loads(tags) if tags else None,
            json.loads(thumbnail_asset) if thumbnail_asset else None
        )
        click.echo("Intelligent playlist updated successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating intelligent playlist: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--name", required=True, help="The name of the live output profile group.")
@click.option("--is-enabled", required=True, type=click.BOOL, help="Indicates if the live output profile group is enabled.")
@click.option("--manifest-type", required=True, help="The manifest type of the live output profile group. The types are HLS, DASH, and BOTH.")
@click.option("--is-default-group", required=True, type=click.BOOL, help="Indicates if the live output profile group is the default group.")
@click.option("--live-output-type", required=True, help="The type of the live output profile group in JSON list format.")
@click.option("--archive-live-output-profile", required=False, help="The archive live output profile of the live output profile group in JSON list format.")
@click.option("--live-output-profiles", required=True, help="The live output profiles of the live output profile group in JSON list format.")
@click.pass_context
def create_live_output_profile_group(ctx, name, is_enabled, manifest_type, is_default_group, live_output_type, archive_live_output_profile, live_output_profiles):
    """Create live output profile group"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.create_live_output_profile_group(
            name,
            is_enabled,
            manifest_type,
            is_default_group,
            json.loads(live_output_type),
            json.loads(archive_live_output_profile) if archive_live_output_profile else None,
            json.loads(live_output_profiles)
        )
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error creating live output profile group: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--event-id", required=True, help="The ID of the event to extend the live schedule of.")
@click.option("--recurring-days", required=True, help="The days of the week to extend the live schedule of in JSON list format.")
@click.option("--recurring-weeks", required=True, type=click.INT, help="The number of weeks to extend the live schedule of.")
@click.option("--end-date", required=False, help="The end date to extend the live schedule of.")
@click.pass_context
def extend_live_schedule(ctx, event_id, recurring_days, recurring_weeks, end_date):
    """Extend live schedule of event"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.extend_live_schedule(
            event_id,
            json.loads(recurring_days),
            recurring_weeks,
            end_date
        )
        click.echo("Live schedule extended successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error extending live schedule: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--content-definition-id", required=True, help="The ID of the content definition to update.")
@click.option("--name", required=False, help="The name of the content definition.")
@click.option("--content-fields", required=False, help="The content fields of the content definition in JSON list format.")
@click.option("--content-definition-group", required=False, help="The content definition group of the content definition.")
@click.option("--content-definition-type", required=False, help="The content definition type of the content definition.")
@click.option("--display-field", required=False, help="The display field of the content definition.")
@click.option("--route-item-name-field", required=False, help="The name of the route item.")
@click.option("--security-groups", required=False, multiple=True, help="The security group(s) of the content definition. Multiple flags for multiple security groups.")
@click.option("--system-roles", required=False, multiple=True, help="The system roles of the content definition. Multiple flags for multiple system roles.")
@click.option("--include-in-tags", required=False, type=click.BOOL, help="Whether to include the content definition in tags.")
@click.option("--index-content", required=False, type=click.BOOL, help="Whether to index the content.")
@click.pass_context
def update_content_definition(ctx, content_definition_id, name, content_fields, content_definition_group, content_definition_type, display_field, route_item_name_field, security_groups, system_roles, include_in_tags, index_content):
    """Update content definition by id"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        content_fields_list = json.loads(content_fields) if content_fields else None

        nomad_sdk.update_content_definition(
            content_definition_id, name, content_fields_list, content_definition_group, content_definition_type,
            display_field, route_item_name_field, security_groups, system_roles, include_in_tags, index_content
        )
        click.echo("Content definition updated successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating content definition: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--content-id", required=True, help="The ID of the content to update.")
@click.option("--content-definition-id", required=True, help="The ID of the content definition the content belongs to.")
@click.option("--properties", required=True, help="The properties to update in JSON dict format.")
@click.option("--language-id", required=False, help="The language id of the asset to upload.")
@click.pass_context
def update_content(ctx, content_id, content_definition_id, properties, language_id):
    """Update content by id"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        properties_dict = json.loads(properties)
        result = nomad_sdk.update_content(content_id, content_definition_id, properties_dict, language_id)
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating content: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--schedule-id", required=True, help="The ID of the schedule the intelligent schedule is to be updated.")
@click.option("--default-video-asset", required=True, help="The default video asset of the intelligent schedule in JSON dict format.")
@click.option("--name", required=False, help="The name of the intelligent schedule.")
@click.option("--thumbnail-asset", required=False, help="The thumbnail asset of the intelligent schedule in JSON dict format.")
@click.option("--time-zone-id", required=False, help="The time zone ID of the intelligent schedule.")
@click.pass_context
def update_intelligent_schedule(ctx, schedule_id, default_video_asset, name, thumbnail_asset, time_zone_id):
    """Update intelligent schedule"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.update_intelligent_schedule(
            schedule_id,
            json.loads(default_video_asset),
            name,
            json.loads(thumbnail_asset) if thumbnail_asset else None,
            time_zone_id
        )
        click.echo("Intelligent schedule updated successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating intelligent schedule: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--schedule-id", required=True, help="The ID of the schedule the schedule item is to be moved from.")
@click.option("--item-id", required=True, help="The ID of the item to be moved.")
@click.option("--previous-item", required=False, help="The previous item of the schedule item.")
@click.pass_context
def move_schedule_item(ctx, schedule_id, item_id, previous_item):
    """Move schedule item"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.move_schedule_item(schedule_id, item_id, previous_item)
        click.echo("Schedule item moved successfully.")
        click.echo(result)

    except Exception as e:
        click.echo(json.dumps({"error": f"Error moving schedule item: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--schedule-id", required=True, help="The ID of the schedule to be stopped.")
@click.option("--force-stop", required=False, type=click.BOOL, help="Whether or not to force a stop.")
@click.pass_context
def stop_schedule(ctx, schedule_id, force_stop):
    """Stop schedule"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.stop_schedule(schedule_id, force_stop)
        click.echo("Schedule stopped successfully.")
        click.echo(result)

    except Exception as e:
        click.echo(json.dumps({"error": f"Error stopping schedule: {e}"}))
        sys.exit(1)


from nomad_media_pip.src.nomad_sdk import Nomad_SDK
from nomad_media_cli.helpers.utils import get_config

import click
import json
import sys

@click.command()
@click.option("--username", required=True, help="Username for authentication")
@click.option("--password", required=True, help="Password for authentication")
@click.pass_context
def login(ctx, username, password):
    """Login to the service"""

    get_config(ctx)

    config = ctx.obj["config"]
    config_path = ctx.obj["config_path"]

    try:
        login_config = config.copy()
        login_config["username"] = username
        login_config["password"] = password
        
        nomad_sdk = Nomad_SDK(login_config)
        nomad_sdk.login()
        
        config["token"] = nomad_sdk.token
        config["refresh_token_val"] = nomad_sdk.refresh_token_val
        config["expiration_seconds"] = nomad_sdk.expiration_seconds
        config["id"] = nomad_sdk.id
        
        with open(config_path, "w") as file:
            json.dump(config, file, indent=4)

        click.echo(json.dumps({ "message": "Successfully logged in." }))
            
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error logging in: Invalid credentials" }))
        sys.exit(1)


from nomad_media_pip.src.nomad_sdk import Nomad_SDK

import click
import json
import os
import sys

def get_config(ctx):
    config_path = ctx.obj["config_path"]    

    try:
        if os.path.exists(config_path):
            with open(config_path, "r") as file:
                config = json.load(file)
            ctx.obj["config"] = config
        else:
            click.echo(json.dumps({ "error": "No configuration found. Please run 'init' first." }))
            sys.exit(1)
    
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error loading configuration: {e}" }))
        sys.exit(1)

def initialize_sdk(ctx):
    get_config(ctx)
    config = ctx.obj["config"]

    try:
        if config is not None:
            if not "token" in config:
                click.echo(json.dumps({ "error": "No token found. Please run 'login' first." }))
                sys.exit(1)            

            nomad_sdk = Nomad_SDK(config)

            nomad_sdk.token = config.get("token")
            nomad_sdk.refresh_token_val = config.get("refresh_token_val")
            
            ctx.obj["nomad_sdk"] = nomad_sdk
            
        else:
            click.echo(json.dumps({ "error": "No configuration found. Please run 'init' first." }))
            sys.exit(1)
            
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error initializing SDK: {e}" }))
        sys.exit(1)


from nomad_media_cli.helpers.utils import initialize_sdk
from nomad_media_cli.helpers.get_id_from_url import get_id_from_url

import click
import json
import sys

@click.command()
@click.option("--id", help="Asset id to list the collections for.")
@click.option("--url", help="The Nomad URL of the Asset (file or folder) to list the collections for (bucket::object-key).")
@click.option("--object-key", help="Object-key of the Asset (file or folder) to list the collections for. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.pass_context
def list_asset_collections(ctx, id, url, object_key):
    """List collections"""    
    
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    if not id and not url and not object_key:
        click.echo(json.dumps({ "error": "Please provide an id, url or object-key" }))
        sys.exit(1)
        
    if url or object_key:
        id = get_id_from_url(ctx, url, object_key, nomad_sdk)

    try:    
        asset_metadata = nomad_sdk.get_asset_details(id)
        
        collections = asset_metadata["collections"]
        collections = [collection["description"] for collection in collections]
        
        click.echo(json.dumps(collections, indent=4))

    except Exception as e:
        click.echo(json.dumps({ "error": f"Error listing collections: {e}" }))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--ids", required=False, multiple=True, help="The ID(s) of the assets to be copied")
@click.option("--urls", required=False, multiple=True, help="The Nomad URL(s) of the Asset (file or folder) to copy the assets for (bucket::object-key).")
@click.option("--object-keys", required=False, multiple=True, help="Object-key(s) of the Asset (file or folder) to copy the assets for. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.option("--destination-folder-id", required=True, help="The destination folder ID of the assets.")
@click.option("--batch-action", required=False, help="The actions to be performed in JSON dict format.")
@click.option("--content-definition-id", required=False, help="The content definition ID of the assets.")
@click.option("--schema-name", required=False, help="The schema name of the assets.")
@click.option("--resolver-exempt", required=False, type=click.BOOL, help="The resolver exempt of the assets.")
@click.pass_context
def copy_asset(ctx, ids, urls, object_keys, destination_folder_id, batch_action, content_definition_id, schema_name, resolver_exempt):
    """Copy asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]
    
    for url in urls:
        if url and "::" not in url:
            click.echo(json.dumps({ "error": "Please provide a valid path or set the default bucket." }))               
            sys.exit(1)
        
        url_search_results = nomad_sdk.search(None, None, None, [{
            "fieldName": "url",
            "operator": "equals",
            "values": url
        }], None, None, None, None, None, None, None, None, None)
        
        if not url_search_results or len(url_search_results["items"] == 0):
            click.echo(json.dumps({ "error": f"URL {url} not found." }))
            sys.exit(1)
            
        ids.append(url_search_results["items"][0]["id"])
        
    for object_key in object_keys:
        if "bucket" in ctx.obj:
            url = f"{ctx.obj['bucket']}::{object_key}"
        else:
            click.echo(json.dumps({ "error": "Please set bucket using `set-bucket` or use url." }))
            sys.exit(1)
        
        url_search_results = nomad_sdk.search(None, None, None, [{
            "fieldName": "url",
            "operator": "equals",
            "values": url
        }], None, None, None, None, None, None, None, None, None)
        
        if not url_search_results or len(url_search_results["items"] == 0):
            click.echo(json.dumps({ "error": f"URL {url} not found." }))
            sys.exit(1)
            
        ids.append(url_search_results["items"][0]["id"])

    try:
        result = nomad_sdk.copy_asset(
            ids,
            destination_folder_id,
            json.loads(batch_action) if batch_action else None,
            content_definition_id,
            schema_name,
            resolver_exempt
        )
        click.echo("Asset copied successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error copying asset: {e}"}))
        sys.exit(1)


from nomad_media_cli.helpers.utils import initialize_sdk
import click
import json
import sys

@click.command()
@click.option("--id", required=True, help="The id of the asset to get the audit for.")
@click.pass_context
def get_audit(ctx, id):
    """Get the audit for an asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk._get_audit(id)
        if result:
            click.echo(json.dumps(result, indent=4))
        else:
            click.echo("No audit information found.")

    except Exception as e:
        click.echo(json.dumps({ "error": f"Error getting audit: {e}" }))
        sys.exit(1)


import click
import json
from nomad_media_cli.helpers.utils import initialize_sdk
from nomad_media_cli.helpers.get_content_definition_id import get_content_definition_id

@click.command()
@click.option("--id", help="The id of the content definition.")
@click.option("--name", help="The display name of the content definition.")
@click.option("--offset", type=click.INT, default=0, help="The offset to start from.")
@click.pass_context
def get_content_definition_contents(ctx, id, name, offset):
    """Gets the contents of a content definition"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]
    
    if id and name:
        click.echo(json.dumps({ "error": "Please provide an id or name." }))
        return

    try:
        if name:
            id = get_content_definition_id(ctx, name)
            if not id:
                click.echo(json.dumps({ "error": f"Content definition with name {name} not found." }))
                return

        result = nomad_sdk.search(None, offset, None, [{
            "fieldName": "contentDefinitionId",
            "operator": "equals",
            "values": id
        }], None, None, None, None, None, None, None, None, None, None, None)
        
        click.echo(json.dumps(result, indent=4))
        
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error getting content definition contents: {e}" }))
        return


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to get the asset child nodes for.")
@click.option("--folder-id", required=True, help="The ID of the folder the asset is in.")
@click.option("--sort-column", required=True, help="The column to sort by.")
@click.option("--is-desc", required=True, type=click.BOOL, help="Whether the sort is descending or not.")
@click.option("--page-index", required=True, type=click.INT, help="The page index of the asset child nodes.")
@click.option("--page-size", required=True, type=click.INT, help="The page size of the asset child nodes.")
@click.pass_context
def get_asset_child_nodes(ctx, id, folder_id, sort_column, is_desc, page_index, page_size):
    """Get asset child nodes"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.get_asset_child_nodes(id, folder_id, sort_column, is_desc, page_index, page_size)
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error getting asset child nodes: {e}"}))
        sys.exit(1)